@@ -1,7 +1,7 @@
 # This file is part of django-popularity.
 # 
 # django-popularity: A generic view- and popularity tracking pluggable for Django. 
-# Copyright (C) 2008-2009 Mathijs de Bruin
+# Copyright (C) 2008-2010 Mathijs de Bruin
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -23,7 +23,6 @@ from datetime import datetime
 from math import log
 
 from django.db import models, connection
-from django.db.models.aggregates import Max
 from django.db.models.expressions import *
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
@@ -107,7 +106,7 @@ class ViewTrackerQuerySet(models.query.QuerySet):
         assert relative_to.__class__ == self.__class__, \
                 'relative_to should be of type %s but is of type %s' % (self.__class__, relative_to.__class__)
             
-        maxviews = relative_to.aggregate(Max('views'))['views__max']
+        maxviews = relative_to.aggregate(models.Max('views'))['views__max']
         
         SQL_RELVIEWS = self._SQL_RELVIEWS % {'maxviews' : maxviews}
         
@@ -191,7 +190,7 @@ class ViewTrackerQuerySet(models.query.QuerySet):
 
         SQL_POPULARITY = self._SQL_POPULARITY % {'age' : _SQL_AGE }
 
-        maxpopularity = relative_to.extra(select={'popularity' : SQL_POPULARITY}).aggregates(Max('popularity'))['popularity__max']
+        maxpopularity = relative_to.extra(select={'popularity' : SQL_POPULARITY}).aggregate(models.Max('popularity'))['popularity__max']
         
         SQL_RELPOPULARITY = self._SQL_RELPOPULARITY % {'popularity'    : SQL_POPULARITY,
                                                        'maxpopularity' : maxpopularity }
@@ -222,7 +221,7 @@ class ViewTrackerQuerySet(models.query.QuerySet):
         
         SQL_POPULARITY = self._SQL_POPULARITY % {'age' : _SQL_AGE }
         
-        maxpopularity = relative_to.extra(select={'popularity' : SQL_POPULARITY}).aggregates(Max('popularity'))['popularity__max']
+        maxpopularity = relative_to.extra(select={'popularity' : SQL_POPULARITY}).aggregate(models.Max('popularity'))['popularity__max']
         
         SQL_RELPOPULARITY = self._SQL_RELPOPULARITY % {'popularity'    : SQL_POPULARITY,
                                                        'maxpopularity' : maxpopularity }
@@ -268,13 +267,13 @@ class ViewTrackerQuerySet(models.query.QuerySet):
         
         assert abs(relview+relage+novelty+relpopularity+random+relevance) > 0, 'You should at least give me something to order by!'
         
-        maxviews = relative_to.aggregates(Max('views'))['views__max']
+        maxviews = relative_to.aggregate(models.Max('views'))['views__max']
         
         SQL_RELVIEWS = self._SQL_RELVIEWS % {'maxviews' : maxviews}
         
         _SQL_AGE = self._SQL_AGE % {'now' : self._get_db_datetime() }
         
-        maxage = relative_to_extra(select={'age':_SQL_AGE}).aggregates(Max('age'))['age__max']
+        maxage = relative_to_extra(select={'age':_SQL_AGE}).aggregate(Max('age'))['age__max']
 
         SQL_RELAGE = self._SQL_RELAGE % {'age'    : _SQL_AGE,
                                          'maxage' : maxage}
@@ -293,7 +292,7 @@ class ViewTrackerQuerySet(models.query.QuerySet):
                                             
         SQL_POPULARITY = self._SQL_POPULARITY % {'age' : _SQL_AGE }
 
-        maxpopularity = relative_to.extra(select={'popularity':SQL_POPULARITY}).aggregates(Max('popularity'))['popularity__max']
+        maxpopularity = relative_to.extra(select={'popularity':SQL_POPULARITY}).aggregate(Max('popularity'))['popularity__max']
 
         SQL_RELPOPULARITY = self._SQL_RELPOPULARITY % {'popularity'    : SQL_POPULARITY,
                                                        'maxpopularity' : maxpopularity }
@@ -375,7 +374,7 @@ class ViewTrackerQuerySet(models.query.QuerySet):
     def get_for_objects(self, objects):
         """ Gets the viewtrackers for specified objects, or creates them 
             if requested. """
-
+        
         qs = self.none()
         for obj in objects:
             ct = ContentType.objects.get_for_model(obj.__class__)
@@ -383,6 +382,35 @@ class ViewTrackerQuerySet(models.query.QuerySet):
             qs = qs | self.filter(content_type=ct, object_id=obj.pk)
         
         return self & qs
+    
+    def get_for_queryset(self, qs):
+        """ Gets the viewtrackers for the objects in a specified queryset. """
+        
+        ct = ContentType.objects.get_for_model(qs.model)
+            
+        return self.filter(content_type=ct, object_id__in=qs.values('pk'))
+    
+    def get_object_list(self):
+        """ Gets a list with all the objects tracked in the current queryset. """
+        
+        obj_list = []
+        for obj in self:
+            obj_list.append(obj.content_object)
+        
+        return obj_list
+    
+    def get_querysets(self):
+        """ Gets a list of all the querysets for the objects tracked in the current queryset. """
+        
+        qs_list = []
+        for ct_id in self.values('content_type').distinct():
+            ct = ContentType.objects.get_for_id(ct_id)
+            qs_inner = self.filter(content_type=ct_id).values('object_id')
+            qs = ct.model_class().objects.filter(pk__in=qs_inner)
+            
+            qs_list.append(qs)
+        
+        return qs_list
 
 class ViewTrackerManager(models.Manager):
     """ Manager methods to do stuff like:
@@ -441,6 +469,12 @@ class ViewTrackerManager(models.Manager):
     
     def get_for_objects(self, *args, **kwargs):
         return self.get_query_set().get_for_objects(*args, **kwargs)
+    
+    def get_for_queryset(self, *args, **kwargs):
+        return self.get_query_set().get_for_queryset(*args, **kwargs)
+    
+    def get_object_list(self, *args, **kwargs):
+        return self.get_query_set().get_object_list(*args, **kwargs)
 
 
 class ViewTracker(models.Model):
